#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.links.interfaces import ILinkContainerTarget, IInternalLink, ILinkContainer, IBaseLink, \
    IExternalLink, IMailtoLink
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION
from pyams_i18n.interfaces import II18n
from pyams_skin.interfaces.viewlet import IToolbarAddingMenu
from pyams_skin.layer import IPyAMSLayer
from pyams_utils.interfaces import VIEW_SYSTEM_PERMISSION

# import packages
from pyams_content.component.links import InternalLink, ExternalLink, MailtoLink
from pyams_content.component.links.zmi.container import LinkContainerView
from pyams_form.form import AJAXAddForm, AJAXEditForm
from pyams_form.security import ProtectedFormObjectMixin
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.viewlet.toolbar import ToolbarMenuItem
from pyams_utils.traversing import get_parent
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogAddForm, AdminDialogEditForm
from pyramid.view import view_config
from z3c.form import field

from pyams_content import _


#
# Internal links views
#

@viewlet_config(name='add-internal-link.menu', context=ILinkContainerTarget, view=LinkContainerView,
                layer=IPyAMSLayer, manager=IToolbarAddingMenu, weight=50)
class InternalLinkAddMenu(ProtectedFormObjectMixin, ToolbarMenuItem):
    """Internal link add menu"""

    label = _("Add internal link")
    label_css_class = 'fa fa-fw fa-link'

    url = 'add-internal-link.html'
    modal_target = True


@pagelet_config(name='add-internal-link.html', context=ILinkContainerTarget, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class InternalLinkAddForm(AdminDialogAddForm):
    """Internal link add form"""

    legend = _("Add new internal link")
    icon_css_class = 'fa fa-fw fa-link'

    fields = field.Fields(IInternalLink).omit('__parent__', '__name__')

    @property
    def ajax_handler(self):
        origin = self.request.params.get('origin')
        if origin == 'link':
            return 'add-internal-link-link.json'
        else:
            return 'add-internal-link.json'

    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(InternalLinkAddForm, self).updateWidgets(prefix)
        if 'description' in self.widgets:
            self.widgets['description'].widget_css_class = 'textarea'

    def create(self, data):
        return InternalLink()

    def add(self, object):
        ILinkContainer(self.context)['none'] = object


@view_config(name='add-internal-link.json', context=ILinkContainerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class InternalLinkAJAXAddForm(AJAXAddForm, InternalLinkAddForm):
    """Internal link add form, JSON renderer"""

    def get_ajax_output(self, changes):
        return {'status': 'reload',
                'location': '#links.html'}


@view_config(name='add-internal-link-link.json', context=ILinkContainerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class InternalLinkLinkAJAXAddForm(AJAXAddForm, InternalLinkAddForm):
    """Internal link link add form, JSON renderer"""

    def get_ajax_output(self, changes):
        target = get_parent(self.context, ILinkContainerTarget)
        container = ILinkContainer(target)
        links = [{'id': link.__name__,
                  'text': II18n(link).query_attribute('title', request=self.request)}
                 for link in container.values()]
        return {'status': 'callback',
                'callback': 'PyAMS_content.links.refresh',
                'options': {'links': links,
                            'new_link': {'id': changes.__name__,
                                         'text': II18n(changes).query_attribute('title', request=self.request)}}}


@pagelet_config(name='properties.html', context=IInternalLink, layer=IPyAMSLayer, permission=VIEW_SYSTEM_PERMISSION)
class InternalLinkPropertiesEditForm(AdminDialogEditForm):
    """Internal link properties edit form"""

    legend = _("Edit internal link properties")
    icon_css_class = 'fa fa-fw fa-link'

    fields = field.Fields(IInternalLink).omit('__parent__', '__name__')
    ajax_handler = 'properties.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(InternalLinkPropertiesEditForm, self).updateWidgets(prefix)
        if 'description' in self.widgets:
            self.widgets['description'].widget_css_class = 'textarea'


@view_config(name='properties.json', context=IInternalLink, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class InternalLinkPropertiesAJAXEditForm(AJAXEditForm, InternalLinkPropertiesEditForm):
    """Internal link properties edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        if ('title' in changes.get(IBaseLink, ())) or \
           ('reference' in changes.get(IInternalLink, ())):
            return {'status': 'reload',
                    'location': '#links.html'}
        else:
            return super(InternalLinkPropertiesAJAXEditForm, self).get_ajax_output(changes)


#
# External links views
#

@viewlet_config(name='add-external-link.menu', context=ILinkContainerTarget, view=LinkContainerView,
                layer=IPyAMSLayer, manager=IToolbarAddingMenu, weight=51)
class ExternalLinkAddMenu(ProtectedFormObjectMixin, ToolbarMenuItem):
    """External link add menu"""

    label = _("Add external link")
    label_css_class = 'fa fa-fw fa-external-link'

    url = 'add-external-link.html'
    modal_target = True


@pagelet_config(name='add-external-link.html', context=ILinkContainerTarget, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class ExternalLinkAddForm(AdminDialogAddForm):
    """External link add form"""

    legend = _("Add new external link")
    icon_css_class = 'fa fa-fw fa-external-link'

    fields = field.Fields(IExternalLink).omit('__parent__', '__name__')

    @property
    def ajax_handler(self):
        origin = self.request.params.get('origin')
        if origin == 'link':
            return 'add-external-link-link.json'
        else:
            return 'add-external-link.json'

    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(ExternalLinkAddForm, self).updateWidgets(prefix)
        if 'description' in self.widgets:
            self.widgets['description'].widget_css_class = 'textarea'

    def create(self, data):
        return ExternalLink()

    def add(self, object):
        ILinkContainer(self.context)['none'] = object


@view_config(name='add-external-link.json', context=ILinkContainerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class ExternalLinkAJAXAddForm(AJAXAddForm, ExternalLinkAddForm):
    """External link add form, JSON renderer"""

    def get_ajax_output(self, changes):
        return {'status': 'reload',
                'location': '#links.html'}


@view_config(name='add-external-link-link.json', context=ILinkContainerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class ExternalLinkLinkAJAXAddForm(AJAXAddForm, ExternalLinkAddForm):
    """External link link add form, JSON renderer"""

    def get_ajax_output(self, changes):
        target = get_parent(self.context, ILinkContainerTarget)
        container = ILinkContainer(target)
        links = [{'id': link.__name__,
                  'text': II18n(link).query_attribute('title', request=self.request)}
                 for link in container.values()]
        return {'status': 'callback',
                'callback': 'PyAMS_content.links.refresh',
                'options': {'links': links,
                            'new_link': {'id': changes.__name__,
                                         'text': II18n(changes).query_attribute('title', request=self.request)}}}


@pagelet_config(name='properties.html', context=IExternalLink, layer=IPyAMSLayer, permission=VIEW_SYSTEM_PERMISSION)
class ExternalLinkPropertiesEditForm(AdminDialogEditForm):
    """External link properties edit form"""

    legend = _("Edit external link properties")
    icon_css_class = 'fa fa-fw fa-external-link'

    fields = field.Fields(IExternalLink).omit('__parent__', '__name__')
    ajax_handler = 'properties.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(ExternalLinkPropertiesEditForm, self).updateWidgets(prefix)
        if 'description' in self.widgets:
            self.widgets['description'].widget_css_class = 'textarea'


@view_config(name='properties.json', context=IExternalLink, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class ExternalLinkPropertiesAJAXEditForm(AJAXEditForm, ExternalLinkPropertiesEditForm):
    """External link properties edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        if ('title' in changes.get(IBaseLink, ())) or \
           ('reference' in changes.get(IExternalLink, ())):
            return {'status': 'reload',
                    'location': '#links.html'}
        else:
            return super(ExternalLinkPropertiesAJAXEditForm, self).get_ajax_output(changes)


#
# Mailto links views
#


@viewlet_config(name='add-mailto-link.menu', context=ILinkContainerTarget, view=LinkContainerView,
                layer=IPyAMSLayer, manager=IToolbarAddingMenu, weight=52)
class MailtoLinkAddMenu(ProtectedFormObjectMixin, ToolbarMenuItem):
    """Mailto link add menu"""

    label = _("Add mailto link")
    label_css_class = 'fa fa-fw fa-envelope-o'

    url = 'add-mailto-link.html'
    modal_target = True


@pagelet_config(name='add-mailto-link.html', context=ILinkContainerTarget, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class MailtoLinkAddForm(AdminDialogAddForm):
    """Mailto link add form"""

    legend = _("Add new mailto link")
    icon_css_class = 'fa fa-fw fa-envelope-o'

    fields = field.Fields(IMailtoLink).omit('__parent__', '__name__')

    @property
    def ajax_handler(self):
        origin = self.request.params.get('origin')
        if origin == 'link':
            return 'add-mailto-link-link.json'
        else:
            return 'add-mailto-link.json'

    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(MailtoLinkAddForm, self).updateWidgets(prefix)
        if 'description' in self.widgets:
            self.widgets['description'].widget_css_class = 'textarea'

    def create(self, data):
        return MailtoLink()

    def add(self, object):
        ILinkContainer(self.context)['none'] = object


@view_config(name='add-mailto-link.json', context=ILinkContainerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class MailtoLinkAJAXAddForm(AJAXAddForm, MailtoLinkAddForm):
    """Mailto link add form, JSON renderer"""

    def get_ajax_output(self, changes):
        return {'status': 'reload',
                'location': '#links.html'}


@view_config(name='add-mailto-link-link.json', context=ILinkContainerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class MailtoLinkLinkAJAXAddForm(AJAXAddForm, MailtoLinkAddForm):
    """Mailto link link add form, JSON renderer"""

    def get_ajax_output(self, changes):
        target = get_parent(self.context, ILinkContainerTarget)
        container = ILinkContainer(target)
        links = [{'id': link.__name__,
                  'text': II18n(link).query_attribute('title', request=self.request)}
                 for link in container.values()]
        return {'status': 'callback',
                'callback': 'PyAMS_content.links.refresh',
                'options': {'links': links,
                            'new_link': {'id': changes.__name__,
                                         'text': II18n(changes).query_attribute('title', request=self.request)}}}


@pagelet_config(name='properties.html', context=IMailtoLink, layer=IPyAMSLayer, permission=VIEW_SYSTEM_PERMISSION)
class MailtoLinkPropertiesEditForm(AdminDialogEditForm):
    """Mailto link properties edit form"""

    legend = _("Edit mailto link properties")
    icon_css_class = 'fa fa-fw fa-envelope-o'

    fields = field.Fields(IMailtoLink).omit('__parent__', '__name__')
    ajax_handler = 'properties.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(MailtoLinkPropertiesEditForm, self).updateWidgets(prefix)
        if 'description' in self.widgets:
            self.widgets['description'].widget_css_class = 'textarea'


@view_config(name='properties.json', context=IMailtoLink, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class MailtoLinkPropertiesAJAXEditForm(AJAXEditForm, MailtoLinkPropertiesEditForm):
    """Mailto link properties edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        if ('title' in changes.get(IBaseLink, ())) or \
           ('reference' in changes.get(IMailtoLink, ())):
            return {'status': 'reload',
                    'location': '#links.html'}
        else:
            return super(MailtoLinkPropertiesAJAXEditForm, self).get_ajax_output(changes)
