#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from hypatia.interfaces import ICatalog
from pyams_content.interfaces import IBaseContentInfo
from pyams_content.interfaces.review import IReviewComments
from pyams_content.shared.common.interfaces import IWfSharedContent, IWfSharedContentRoles, ISharedContent, ISharedTool
from pyams_security.interfaces import IDefaultProtectionPolicy
from pyams_sequence.interfaces import ISequentialIdTarget, ISequentialIdInfo
from pyams_utils.interfaces import VIEW_PERMISSION
from pyams_workflow.interfaces import IWorkflowPublicationSupport, IWorkflow, IObjectClonedEvent, IWorkflowVersions
from pyramid.interfaces import IWSGIApplicationCreatedEvent
from zope.dublincore.interfaces import IZopeDublinCore
from zope.intid.interfaces import IIntIds
from zope.lifecycleevent.interfaces import IObjectModifiedEvent

# import packages
from persistent import Persistent
from pyams_i18n.content import I18nManagerMixin
from pyams_security.property import RolePrincipalsFieldProperty
from pyams_security.security import ProtectedObject
from pyams_security.utility import get_principal
from pyams_utils.adapter import adapter_config, ContextAdapter
from pyams_utils.date import format_datetime
from pyams_utils.registry import query_utility
from pyams_utils.request import query_request, check_request
from pyams_utils.timezone import tztime
from pyams_utils.traversing import get_parent
from pyams_utils.vocabulary import vocabulary_config
from pyramid.events import subscriber
from pyramid.settings import asbool
from pyramid.threadlocal import get_current_registry
from zope.container.contained import Contained
from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm

from pyams_content import _


#
# Content types management
#

CONTENT_TYPES = {}


def register_content_type(content):
    """Register a new content type"""
    CONTENT_TYPES[content.content_type] = content


@subscriber(IWSGIApplicationCreatedEvent)
def handle_content_types(event):
    """Check for content types to un-register"""
    registry = get_current_registry()
    for key, content in CONTENT_TYPES.copy().items():
        if not asbool(registry.settings.get('pyams_content.register.{0}'.format(key), True)):
            del CONTENT_TYPES[key]


@vocabulary_config(name='PyAMS content types')
class ContentTypesVocabulary(SimpleVocabulary):
    """Content types vocabulary"""

    def __init__(self, context):
        request = check_request()
        translate = request.localizer.translate
        terms = [SimpleTerm(content.content_type, title=translate(content.content_name))
                 for content in CONTENT_TYPES.values()]
        super(ContentTypesVocabulary, self).__init__(terms)


#
# Workflow shared content class and adapters
#

@implementer(IDefaultProtectionPolicy, IWfSharedContent, IWfSharedContentRoles, IWorkflowPublicationSupport)
class WfSharedContent(ProtectedObject, Persistent, Contained, I18nManagerMixin):
    """Shared data content class"""

    __roles__ = ('pyams.Owner', 'pyams.Manager', 'pyams.Contributors', 'pyams.Reader', 'pyams.Guest')
    roles_interface = IWfSharedContentRoles

    owner = RolePrincipalsFieldProperty(IWfSharedContentRoles['owner'])
    managers = RolePrincipalsFieldProperty(IWfSharedContentRoles['managers'])
    contributors = RolePrincipalsFieldProperty(IWfSharedContentRoles['contributors'])
    readers = RolePrincipalsFieldProperty(IWfSharedContentRoles['readers'])
    guests = RolePrincipalsFieldProperty(IWfSharedContentRoles['guests'])

    content_type = None

    title = FieldProperty(IWfSharedContent['title'])
    short_name = FieldProperty(IWfSharedContent['short_name'])
    creator = FieldProperty(IWfSharedContent['creator'])
    modifiers = FieldProperty(IWfSharedContent['modifiers'])
    last_modifier = FieldProperty(IWfSharedContent['last_modifier'])
    description = FieldProperty(IWfSharedContent['description'])
    keywords = FieldProperty(IWfSharedContent['keywords'])
    notepad = FieldProperty(IWfSharedContent['notepad'])

    @property
    def first_owner(self):
        versions = IWorkflowVersions(self)
        return versions.get_version(1).creator

    @property
    def creation_label(self):
        request = check_request()
        translate = request.localizer.translate
        return translate(_('{date} by {principal}')).format(
            date=format_datetime(tztime(IBaseContentInfo(self).created_date)),
            principal=get_principal(request, self.creator).title)

    @property
    def last_update_label(self):
        request = check_request()
        translate = request.localizer.translate
        return translate(_('{date} by {principal}')).format(
            date=format_datetime(tztime(IBaseContentInfo(self).modified_date)),
            principal=get_principal(request, self.last_modifier).title)


@subscriber(IObjectModifiedEvent, context_selector=IWfSharedContent)
def handle_modified_shared_content(event):
    """Define content's modifiers when content is modified"""
    request = query_request()
    if request is not None:
        content = event.object
        principal_id = request.principal.id
        modifiers = content.modifiers or set()
        if principal_id not in modifiers:
            modifiers.add(principal_id)
            content.modifiers = modifiers
            catalog = query_utility(ICatalog)
            intids = query_utility(IIntIds)
            catalog['modifiers'].reindex_doc(intids.register(content), content)
        content.last_modifier = principal_id


@subscriber(IObjectClonedEvent, context_selector=IWfSharedContent)
def handle_cloned_shared_content(event):
    """Handle cloned object when a new version is created

    Current principal is set as version creator, and is added to version
    contributors if he is not the original content's owner
    """
    request = query_request()
    principal_id = request.principal.id
    content = event.object
    content.creator = principal_id
    if principal_id != content.owner:
        # creation of new versions doesn't change owner
        # but new creators are added to contributors list
        contributors = content.contributors or set()
        contributors.add(principal_id)
        content.contributors = contributors
    # reset modifiers
    content.modifiers = set()
    # clear review comments
    IReviewComments(content).clear()


@adapter_config(context=IWfSharedContent, provides=ISequentialIdInfo)
def WfSharedContentSequenceAdapter(context):
    """Shared content sequence adapter"""
    parent = get_parent(context, ISharedContent)
    return ISequentialIdInfo(parent)


@adapter_config(context=IWfSharedContent, provides=IBaseContentInfo)
class WfSharedContentInfoAdapter(ContextAdapter):
    """Shared content base info adapter"""

    @property
    def created_date(self):
        return IZopeDublinCore(self.context).created

    @property
    def modified_date(self):
        return IZopeDublinCore(self.context).modified


@adapter_config(context=IWfSharedContent, provides=IWorkflow)
def WfSharedContentWorkflowAdapter(context):
    """Shared content workflow adapter"""
    parent = get_parent(context, ISharedTool)
    return query_utility(IWorkflow, name=parent.shared_content_workflow)


#
# Main shared content class and adapters
#

@implementer(ISharedContent, ISequentialIdTarget)
class SharedContent(Persistent, Contained):
    """Workflow managed shared data"""

    view_permission = VIEW_PERMISSION

    sequence_name = ''  # use default sequence generator
    sequence_prefix = ''

    @property
    def workflow_name(self):
        return get_parent(self, ISharedTool).shared_content_workflow


@adapter_config(context=ISharedContent, provides=IBaseContentInfo)
class SharedContentInfoAdapter(ContextAdapter):
    """Shared content base info adapter"""

    @property
    def created_date(self):
        return IZopeDublinCore(self.context).created

    @property
    def modified_date(self):
        return IZopeDublinCore(self.context).modified


@adapter_config(context=ISharedContent, provides=IWorkflow)
def SharedContentWorkflowAdapter(context):
    """Shared content workflow adapter"""
    parent = get_parent(context, ISharedTool)
    return query_utility(IWorkflow, name=parent.shared_content_workflow)
